import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Run filtering and create report of filtered-out contacts.
 * @author Pawel P. Wozniak
 */
public class Filters {
	public static void main(String[] args) {
		try
		{
			String ssPredPath = "/home/moher/CS_2_3D/gplmdca_Skwark/Data/SSandACC_4_OriginalFasta/1I5GA.out.ss"; //path to the file with SSpro results
			String accPredPath = "/home/moher/CS_2_3D/gplmdca_Skwark/Data/SSandACC_4_OriginalFasta/1I5GA.out.acc20"; //path to the file with ACCpro20 results
			String fastaPath = "/home/moher/CS_2_3D/gplmdca_Skwark/Data/Original_gplmdca_dataset/1I5G_A.fas"; //path to the FASTA file (first from the file will be read)
			String aliPath = "/home/moher/CS_2_3D/gplmdca_Skwark/Data/Original_gplmdca_dataset/1I5G_A.fas"; //path to the file with alignment
			String resPath = "/home/moher/CS_2_3D/gplmdca_Skwark/Data/Original_gplmdca_dataset/1I5G_A.gplmdca"; //path to the file with DCA results
			String upperTitle = "1I5GA"; //subtitle for the file with filter output
			
			//default filters parameters (below): 
			String notAllowed = "C"; //these secondary structure will not be taken under consideration when list of secondary structure fragments is created for a sequence
			int anyfragSize = 2; //only secondary structure fragments of minimally that length will be used when list of secondary structure fragments is created for a sequence
			
			boolean ifIntraSS = false; //turn on/off (true/false) the intra-secondary structure filter
			String whichSS = "HE"; //which secondary structure elements should be checked by intra-secondary structure filter
			int fragSizeIntra = 5; //value of Sintra parameter for intra-secondary structure filter
			
			boolean ifInterNextSS = false; //turn on/off (true/false) the inter-secondary structure filter
			int interSSsep = 6; //value of Sinter parameter for inter-secondary structure filter
			int loopSize = 1; //value of Sloop parameter for inter-secondary structure filter
			int endSize = 6; //value of e parameter for inter-secondary structure filter
			
			boolean ifBetweenEnds = false; //turn on/off (true/false) the terminus filter
			boolean ifDiffSS = false; //true if contacts between one end of the protein and the rest should be between different secondary structures to be filtered-out
			boolean ifDiffSSBegEnd = false; //True if contacts between two ends of the protein should be between different secondary structures to be filtered-out
			int howMany = 15; //value of t parameter for terminus filter
			boolean ifACCpasser = true; //True if TRSA parameter should be used for terminus filter
			boolean ifBothACC = false; //True if both residues in pair should be in agreement with TRSA parameter
			double accThresLE = 35; //value of TRSA parameter for terminus filter
			
			boolean ifEntropy = false; //turn on/off (true/false) the low entropy filter
			boolean ifBoth = false; //True if both residues in pair should be in agreement with Tent parameter
			double entCutoff = 0.02; //value of Tent parameter for low entropy filter
			boolean ifGaps = true; //True if gaps should be counted as symbols during entropy calculation

			boolean ifGapsFreq = true; //turn on/off (true/false) the frequent gap filter
			double gapFreq = 95; //value of Tgaps parameter for frequent gap filter
			boolean ifBothGapsFreq = false; //True if both residues in pair should be in agreement with Tgaps parameter
			
			boolean ifIntraSSMarks = false; //turn on/off (true/false) the secondary structure straddling filter
			
			boolean ifConservMarks = false; //turn on/off (true/false) the highly conserved filter
			boolean ifConservGaps = true; //True if gaps should be counted as symbols during conservation calculation
			
			boolean ifFirstCysMarks = false; //turn on/off (true/false) the the non-strongest Cys-Cys filter
			
			boolean ifSeqSep = false; //turn on/off (true/false) the inter-terminus filter
			double seqSep = 0.95; //value of Snt parameter for inter-terminus filter
			
			boolean ifBurExpDCA = false; //turn on/off (true/false) the buried-exposed filter
			double aBE = 20; //value of a parameter for buried-exposed filter
			double bBE = 120; //value of b parameter for buried-exposed filter
						
			int top = 200; //number of predicted contacts N to apply filtration on (if mult=0 and div=0)
			double mult = 0; //if number of predicted contacts to apply filtration on depends on sequence length then e.g. 3L/2 == mult*L/div
			double div = 0; //if number of predicted contacts to apply filtration on depends on sequence length then e.g. 3L/2 == mult*L/div
			
			int separation = 5; //separation parameter for contact definition
			//default filters parameters (above): 
			
			//creating title with number of N and name of selected filter (below):
			String titleSuff = upperTitle;
			if(mult > 0)
			{
				titleSuff = titleSuff + "_topM" + ((int)mult) + "D" + ((int)div);
			}
			else
			{
				titleSuff = titleSuff + "_top" + top;
			}
			
			if(ifIntraSS)
			{
				titleSuff = titleSuff + "_intra-secondary-structure_Sintra" + fragSizeIntra;
			}
			if(ifInterNextSS)
			{
				titleSuff = titleSuff + "_inter-secondary-structure_Sloop" + loopSize + "_Sinter" + interSSsep + "_e" + endSize;
			}
			if(ifBetweenEnds)
			{
				titleSuff = titleSuff + "_terminus_t" + howMany + "_TRSA" + accThresLE;
			}
			if(ifEntropy)
			{
				titleSuff = titleSuff + "_low-entropy_Tent" + entCutoff;
			}
			if(ifIntraSSMarks)
			{
				titleSuff = titleSuff + "_secondary-structure-straddling";
			}
			if(ifConservMarks)
			{
				titleSuff = titleSuff + "_highly-conserved";
			}
			if(ifFirstCysMarks)
			{
				titleSuff = titleSuff + "_non-strongest-Cys-Cys";
			}
			if(ifGapsFreq)
			{
				titleSuff = titleSuff + "_frequent-gap_Tgaps" + gapFreq;
			}
			if(ifSeqSep)
			{
				titleSuff = titleSuff + "_inter-terminus_Snt" + seqSep;
			}
			if(ifBurExpDCA)
			{
				titleSuff = titleSuff + "_buried-exposed_a" + aBE + "_b" + bBE;
			}
			//creating title with number of N and name of selected filter (above):

			//initialize writer and list for filtered-out contacts:
			FileWriter writerReport = new FileWriter("raport_" + titleSuff);
			List<String> raport = new ArrayList<String>();
			
			//get FASTA sequence (first from the file):
			String fastaSeq = new FastaFileReader(fastaPath, 1).getFastaSeqs().get(0);
			
			//get number of of predicted contacts N to apply filtration on:
			int topTmp = top;
			if(mult > 0)
			{
				topTmp = (int) ((mult*fastaSeq.length())/div);
			}
			
			//run filtering:
			createReport(resPath, topTmp, separation, notAllowed,
					ssPredPath, accPredPath,
					fastaSeq, fragSizeIntra, anyfragSize, whichSS,
					raport, ifIntraSS,
					ifInterNextSS, interSSsep, loopSize, endSize,
					ifBetweenEnds, ifDiffSS, ifDiffSSBegEnd, howMany,
					ifEntropy, ifBoth, entCutoff, ifGaps, aliPath,
					ifGapsFreq, ifBothGapsFreq, gapFreq,
					ifIntraSSMarks, ifConservMarks, ifConservGaps, ifFirstCysMarks,
					ifSeqSep, seqSep,
					ifACCpasser, ifBothACC, accThresLE,
					ifBurExpDCA, aBE, bBE);
			
			//create file with filtered-out contacts:
			for (int j = 0; j < raport.size(); j++) {
				writerReport.write(raport.get(j) + "\n");
			}
			writerReport.close();
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}

	/**
	 * Run filtering and create report of filtered-out contacts.
	 * @param resPath - path to the file with DCA results
	 * @param top - number of of predicted contacts N to apply filtration on
	 * @param separation - separation parameter for contact definition
	 * @param notAllowed - these secondary structure will not be taken under consideration when list of secondary structure fragments is created for a sequence
	 * @param ssPredPath - path to the file with SSpro results
	 * @param accPredPath - path to the file with ACCpro20 results
	 * @param fastaSeq - FASTA sequence
	 * @param fragSizeIntra - value of Sintra parameter for intra-secondary structure filter
	 * @param anyfragSize - only secondary structure fragments of minimally that length will be used when list of secondary structure fragments is created for a sequence
	 * @param whichSS - which secondary structure elements should be checked by intra-secondary structure filter
	 * @param raport - variable which will contain the list of filtered-out contacts
	 * @param ifIntraSS - turn on/off (true/false) the intra-secondary structure filter
	 * @param ifInterNextSS - turn on/off (true/false) the inter-secondary structure filter
	 * @param interSSsep - value of Sinter parameter for inter-secondary structure filter
	 * @param loopSize - value of Sloop parameter for inter-secondary structure filter
	 * @param endSize - value of e parameter for inter-secondary structure filter
	 * @param ifBetweenEnds - turn on/off (true/false) the terminus filter
	 * @param ifDiffSS - true if contacts between one end of the protein and the rest should be between different secondary structures to be filtered-out
	 * @param ifDiffSSBegEnd - True if contacts between two ends of the protein should be between different secondary structures to be filtered-out
	 * @param howMany - value of t parameter for terminus filter
	 * @param entCutoff - value of Tent parameter for low entropy filter
	 * @param ifBoth - True if both residues in pair should be in agreement with Tent parameter
	 * @param ifEntropy - turn on/off (true/false) the low entropy filter
	 * @param ifGaps - True if gaps should be counted as symbols during entropy calculation
	 * @param aliPath - path to the file with alignment
	 * @param ifGapsFreq - turn on/off (true/false) the frequent gap filter
	 * @param ifBothGapsFreq - True if both residues in pair should be in agreement with Tgaps parameter
	 * @param gapFreq - value of Tgaps parameter for frequent gap filter
	 * @param ifIntraSSMarks - turn on/off (true/false) the secondary structure straddling filter
	 * @param ifConservMarks - turn on/off (true/false) the highly conserved filter
	 * @param ifConservGaps - True if gaps should be counted as symbols during conservation calculation
	 * @param ifFirstCysMarks - turn on/off (true/false) the the non-strongest Cys-Cys filter
	 * @param ifSeqSep - turn on/off (true/false) the inter-terminus filter
	 * @param seqSep - value of Snt parameter for inter-terminus filter
	 * @param ifACCpasser - True if TRSA parameter should be used for terminus filter
	 * @param ifBothACC - True if both residues in pair should be in agreement with TRSA parameter
	 * @param accThres - value of TRSA parameter for terminus filter
	 * @param ifBurExpDCA - turn on/off (true/false) the buried-exposed filter
	 * @param aBE - value of a parameter for buried-exposed filter
	 * @param bBE - value of b parameter for buried-exposed filter
	 */	
	private static void createReport(String resPath, int top, int separation, String notAllowed,
			String ssPredPath, String accPredPath, String fastaSeq, int fragSizeIntra, int anyfragSize,
			String whichSS, List<String> raport,
			boolean ifIntraSS, boolean ifInterNextSS, int interSSsep, int loopSize, int endSize,
			boolean ifBetweenEnds, boolean ifDiffSS, boolean ifDiffSSBegEnd, int howMany,
			boolean ifEntropy, boolean ifBoth, double entCutoff, boolean ifGaps, String aliPath,
			boolean ifGapsFreq, boolean ifBothGapsFreq, double gapFreq,
			boolean ifIntraSSMarks, boolean ifConservMarks, boolean ifConservGaps, boolean ifFirstCysMarks,
			boolean ifSeqSep, double seqSep,
			boolean ifACCpasser, boolean ifBothACC, double accThres,
			boolean ifBurExpDCA, double aBE, double bBE) {
		try
		{
			//Get sorted results from DCA file:
			TopMethods topMeth = new TopMethods();
			topMeth.gatherSortedGplmdca(resPath, 0, 1, 2, ",");
			int[] gdcaSortInd = topMeth.getGdcaSortInd();
			double[] gdcaSortVal = topMeth.getGdcaSortVal();
			List<Integer> posDca1 = topMeth.getPosDca1();
			List<Integer> posDca2 = topMeth.getPosDca2();

			//counters for number of residue pairs to apply filtering on:
			int counter = 0;
			int counterOriTop = 0;
			
			//list of secondary structure fragments:
			List<int[]> frags = new ArrayList<int[]>();
			List<String> fragSS = new ArrayList<String>();
			
			//getting data from SSpro and ACCpro20:
			List<Character> ssPred = null;
			List<Integer> accPred = null;
			File ssPredFile = new File(ssPredPath);
			File accPredFile = new File(accPredPath);
			if(ssPredFile.exists())
			{
				ssPred = ScratchReader.ssRead(ssPredFile.getAbsolutePath(), null);
				accPred = ScratchReader.accRead(accPredFile.getAbsolutePath(), null);
			}
			else
			{
				System.out.println("No " + ssPredPath + " !!!");
			}
			
			if((ssPred != null) && (accPred != null))
			{
				//get list of secondary structure fragments:
				getFragsForPredicted(ssPredPath, fastaSeq, anyfragSize, frags, fragSS, notAllowed, ssPred);
				
				//reading MSA if needed (for entropy, gaps, and conservation filters):
				FastaFileReader fastAliReader = null;
				MSAanalysis msaAnalysis = null;
				if(ifEntropy || ifGapsFreq || ifConservMarks)
				{
					fastAliReader = new FastaFileReader(aliPath, 1000000);
					msaAnalysis = new MSAanalysis(fastAliReader.getFastaSeqs());
				}
				
				//list of Cys-Cys pairs for non-strongest Cys-Cys filter
				List<String> alreadyFound = null;
				if(ifFirstCysMarks)
				{
					alreadyFound = new ArrayList<String>();
				}
				
				//reading strongest N predicted contacts:
				double maxDca = -1000;
				double minDca = 1000;
				for(int i=0; i<gdcaSortInd.length; i++)
				{					
					int pos1 = posDca1.get(gdcaSortInd[i]);
					int pos2 = posDca2.get(gdcaSortInd[i]);
					String aa1 = String.valueOf(fastaSeq.charAt(pos1-1));
					String aa2 = String.valueOf(fastaSeq.charAt(pos2-1));
					double dcaScore = gdcaSortVal[i];
					
					boolean pass = true;
					boolean passInterNextSS = true;
					
					//run the non-strongest Cys-Cys filter:
					if(ifFirstCysMarks)
					{
						pass = ifFirstCysMarksPass(pos1, pos2, aa1, aa2, alreadyFound);
					}
					
					if(Math.abs(pos1-pos2) >= separation)
					{
						if(counterOriTop == 0 && maxDca == -1000)
						{
							maxDca = dcaScore;
						}
						
						if(counterOriTop == top && minDca == 1000)
						{
							minDca = dcaScore;
						}
						
						if(pass)
						{
							//run intra-secondary structure filter:
							if(ifIntraSS)
							{
								pass = ifPassIntraSS(frags, fragSS, pos1, pos2, whichSS, fragSizeIntra);
							}
							
							if(pass)
							{
								//run inter-secondary structure filter:
								if(ifInterNextSS)
								{
									pass = ifPassInterNextSS(frags, pos1, pos2, interSSsep, loopSize, endSize);
								}
								
								if(pass)
								{
									//run terminus filter:
									if(ifBetweenEnds)
									{
										boolean pass1 = getNmbOfContacts(pos1, pos2, 1, howMany, 1, fastaSeq.length(), ifDiffSS, ssPred);
										boolean pass2 = getNmbOfContacts(pos1, pos2, fastaSeq.length()-howMany+1, fastaSeq.length(), 1, fastaSeq.length(), ifDiffSS, ssPred);
										boolean pass3 = getNmbOfContacts(pos1, pos2, 1, howMany, fastaSeq.length()-howMany+1, fastaSeq.length(), ifDiffSSBegEnd, ssPred);
										
										if(!pass1 || !pass2 || !pass3)
										{
											pass = false;
											passInterNextSS = false;
										}
									}
									
									if(pass)
									{
										//run low-entropy filter:
										if(ifEntropy)
										{
											pass = ifEntropyPass(msaAnalysis, pos1, pos2, entCutoff, ifBoth, ifGaps);
										}
										
										if(pass)
										{
											//run frequent gaps filter:
											if(ifGapsFreq)
											{
												pass = ifGapsFreqPass(msaAnalysis, pos1, pos2, ifBothGapsFreq, gapFreq);
											}
											
											if(pass)
											{
												//run secondary structure straddling filter:
												if(ifIntraSSMarks)
												{
													pass = ifPassIntraSSMarks(frags, fragSS, pos1, pos2, separation);
												}
												
												if(pass)
												{
													//run highly conserved filter:
													if(ifConservMarks)
													{
														pass = ifConservMarksPass(msaAnalysis, pos1, pos2, aa1, aa2, ifConservGaps);
													}
													
													if(pass)
													{
														//run inter-terminus filter:
														if(ifSeqSep)
														{
															pass = ifSeqSepPass(pos1, pos2, seqSep, fastaSeq.length());
														}
														
														if(pass)
														{
															//run buried-exposed filter:
															if(ifBurExpDCA)
															{
																pass = ifBurExpDCAPass(pos1, pos2, accPred, aBE, bBE, dcaScore);
															}
														}
													}
												}
											}
										}
									}
								}
							}
						}
						
						//apply TRSA for terminus filter:
						if(ifACCpasser && !passInterNextSS)
						{
							pass = ifACCPass(pos1, pos2, accPred, accThres, ifBothACC);
						}
						
						counterOriTop++;
						if(pass)
						{
							counter++;
						}
						
						if(!pass)
						{
							raport.add(pos1 + " " + pos2 + " " + dcaScore + "     " + maxDca + " " + counterOriTop);
						}
						
						if(counter >= top)
						{
							break;
						}
					}
				}
				raport.add("MinDCA: " + minDca);
			}
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}

	private static boolean ifBurExpDCAPass(int pos1, int pos2, List<Integer> accPred, double aBE, double bBE, double dcaScore) {
		double diff = Math.abs(accPred.get(pos1-1) - accPred.get(pos2-1));
		double y = bBE*dcaScore+aBE;
		if(diff > y)
		{
			return false;
		}
		return true;
	}

	private static boolean ifACCPass(int pos1, int pos2, List<Integer> accPred, double accThres, boolean ifBoth) {
		if((ifBoth && accPred.get(pos1-1) <= accThres && accPred.get(pos2-1) <= accThres) || (!ifBoth && (accPred.get(pos1-1) <= accThres || accPred.get(pos2-1) <= accThres)))
		{
			return true;
		}
		return false;
	}

	private static boolean ifSeqSepPass(int pos1, int pos2, double seqSep, int seqLen) {
		int separation = Math.abs(pos2-pos1);
		double normSepr = ((double)separation)/seqLen;
		if(normSepr <= 1)
		{
			if(normSepr > seqSep)
			{
				return false;
			}
		}
		else
		{
			System.out.println("Normalized sequence separation higher than 1!!!");
		}
		return true;
	}

	private static boolean ifFirstCysMarksPass(int pos1, int pos2, String aa1, String aa2, List<String> alreadyFound) {
		boolean ifFirstFound = false;
		boolean ifSecondFound = false;
		if(aa1.equals("C") && aa2.equals("C"))
		{
			ifFirstFound = checkIfFound(alreadyFound, String.valueOf(pos1)+aa1);
			ifSecondFound = checkIfFound(alreadyFound, String.valueOf(pos2)+aa2);
			if(!ifFirstFound)
			{
				alreadyFound.add(String.valueOf(pos1)+aa1);
			}
			if(!ifSecondFound)
			{
				alreadyFound.add(String.valueOf(pos2)+aa2);
			}
			if(ifFirstFound || ifSecondFound)
			{
				return false;
			}
		}
		return true;
	}

	private static boolean checkIfFound(List<String> alreadyFound, String toFind) {
		for (Iterator<String> iterator = alreadyFound.iterator(); iterator.hasNext();) {
			String whatnow = iterator.next();
			if(whatnow.equals(toFind))
			{
				return true;
			}
		}
		return false;
	}

	private static boolean ifConservMarksPass(MSAanalysis msaAnalysis, int pos1, int pos2, String aa1, String aa2, boolean ifConservGaps) {
		if(!(aa1.equals("C") && aa2.equals("C")))
		{
			double var1 = msaAnalysis.variabilityCalc(pos1-1, 95, ifConservGaps);
			double var2 = msaAnalysis.variabilityCalc(pos2-1, 95, ifConservGaps);
			if(var1 != 0 && var2 != 0)
			{
				return false;
			}
		}
		return true;
	}

	private static boolean ifPassIntraSSMarks(List<int[]> frags, List<String> fragSS, int pos1, int pos2, int separation) {
		if(Math.abs(pos1-pos2) >= separation)
		{
			for (int i = 0; i < frags.size(); i++) {
				if(fragSS.get(i).contains("H"))
				{
					if((pos1 >= frags.get(i)[0]-3 && pos1 <= frags.get(i)[1] && pos2 >= frags.get(i)[0]-3 && pos2 <= frags.get(i)[1]) ||
						((pos1 == frags.get(i)[0]-1 && pos2 == frags.get(i)[1]+1) || (pos2 == frags.get(i)[0]-1 && pos1 == frags.get(i)[1]+1)))
					{
						return false;
					}
				}
				else if(fragSS.get(i).contains("E"))
				{
					if(
						(pos1 >= frags.get(i)[0]-4 && pos1 <= frags.get(i)[1] && pos2 >= frags.get(i)[0]-4 && pos2 <= frags.get(i)[1]) ||
						((pos1 == frags.get(i)[0]-1 && pos2 == frags.get(i)[1]+1) || (pos2 == frags.get(i)[0]-1 && pos1 == frags.get(i)[1]+1)) ||
						(
							Math.abs(frags.get(i)[0]-frags.get(i)[1]) >= 5 && 
							(
								(pos1 == frags.get(i)[0]-2 && pos2 == frags.get(i)[1]+2) || (pos2 == frags.get(i)[0]-2 && pos1 == frags.get(i)[1]+2)
							)
						)
					  )
					{
						return false;
					}
				}
			}
		}
		return true;
	}

	private static boolean ifGapsFreqPass(MSAanalysis msaAnalysis, int pos1, int pos2, boolean ifBothGapsFreq, double gapFreq) {
		boolean gap1 = !msaAnalysis.checkIfNonDeletion(pos1-1, gapFreq);
		boolean gap2 = !msaAnalysis.checkIfNonDeletion(pos2-1, gapFreq);
		
		if((ifBothGapsFreq && gap1 && gap2) || (!ifBothGapsFreq && (gap1 || gap2)))
		{
			return false;
		}

		return true;
	}
	
	private static boolean ifEntropyPass(MSAanalysis msaAnalysis, int pos1, int pos2, double entCutoff, boolean ifBoth, boolean ifGaps) {
		double ent1 = msaAnalysis.shannonEntropyCalc(pos1-1, ifGaps, true);
		double ent2 = msaAnalysis.shannonEntropyCalc(pos2-1, ifGaps, true);
		
		if(ifBoth)
		{
			if(ent1 < entCutoff && ent2 < entCutoff)
			{
				return false;
			}
		}
		else
		{
			if(ent1 < entCutoff || ent2 < entCutoff)
			{
				return false;
			}
		}
		return true;
	}
	
	private static boolean getNmbOfContacts(int pos1, int pos2, int s1, int e1, int s2, int e2, boolean ifDiffSS, List<Character> ssPred) {
		if((pos1 >= s1 && pos1 <= e1 && pos2 >= s2 && pos2 <= e2) || (pos2 >= s1 && pos2 <= e1 && pos1 >= s2 && pos1 <= e2))
		{
			if(!ifDiffSS || ssPred.get(pos1-1) != ssPred.get(pos2-1))
			{
				return false;
			}
		}
		return true;
	}
	
	private static void getFragsForPredicted(String ssPredPath, String fastaSeq, int fragSize, List<int[]> frags, List<String> fragSS, String notAllowed, List<Character> ssPred)
	{
		File ssPredFile = new File(ssPredPath);
		if(fastaSeq.length() == ssPred.size())
		{
			int counter = 0;
			int firstNmb = 0;
			char prevSS = '@';
			for (int i = 0; i < ssPred.size(); i++) {
				if(prevSS != ssPred.get(i))
				{
					if(notAllowed.isEmpty() || (!notAllowed.contains(String.valueOf(prevSS))))
					{
						if(counter >= fragSize)
						{
							int[] tmpFrag = new int[2];
							tmpFrag[0] = firstNmb;
							tmpFrag[1] = i;
							frags.add(tmpFrag);
							fragSS.add(String.valueOf(prevSS));
						}
					}
					prevSS = ssPred.get(i);
					counter = 1;
					firstNmb = i+1;
				}
				else
				{
					counter++;
				}
			}
			
			if(notAllowed.isEmpty() || (!notAllowed.contains(String.valueOf(prevSS))))
			{
				if(counter >= fragSize)
				{
					int[] tmpFrag = new int[2];
					tmpFrag[0] = firstNmb;
					tmpFrag[1] = ssPred.size();
					frags.add(tmpFrag);
					fragSS.add(String.valueOf(prevSS));
				}
			}			
		}
		else
		{
			System.out.println("Wrong SS length: " + ssPredFile.getName().split("\\.")[0]);
		}
	}

	private static boolean ifPassIntraSS(List<int[]> frags, List<String> fragSS, int pos1, int pos2, String whichSS, int fragSizeIntra) {
		if(Math.abs(pos1-pos2) >= fragSizeIntra)
		{
			for (int i = 0; i < frags.size(); i++) {
				if(whichSS.contains(fragSS.get(i)) && Math.abs(frags.get(i)[1]-frags.get(i)[0]) >= fragSizeIntra)
				{
					if(pos1 >= frags.get(i)[0] && pos1 <= frags.get(i)[1] && pos2 >= frags.get(i)[0] && pos2 <= frags.get(i)[1])
					{
						return false;
					}
				}
			}
		}
		return true;
	}
	
	private static boolean ifPassInterNextSS(List<int[]> frags, int pos1, int pos2, int interSSsep, int loopSize, int endSize) {
		if(Math.abs(pos1-pos2) >= interSSsep)
		{
			for (int i = 1; i < frags.size(); i++) {
				if((frags.get(i)[0]-frags.get(i-1)[1] <= (loopSize+1)) && 
						((pos1 > frags.get(i-1)[1]-endSize && pos1 <= frags.get(i-1)[1] && pos2 > frags.get(i)[1]-endSize && pos2 <= frags.get(i)[1]) ||
						(pos2 > frags.get(i-1)[1]-endSize && pos2 <= frags.get(i-1)[1] && pos1 > frags.get(i)[1]-endSize && pos1 <= frags.get(i)[1]) ||
						(pos1 < frags.get(i-1)[0]+endSize && pos1 >= frags.get(i-1)[0] && pos2 < frags.get(i)[0]+endSize && pos2 >= frags.get(i)[0]) ||
						(pos2 < frags.get(i-1)[0]+endSize && pos2 >= frags.get(i-1)[0] && pos1 < frags.get(i)[0]+endSize && pos1 >= frags.get(i)[0])))
				{
					return false;
				}
			}
			return true;
		}
		return true;
	}
}
